@@ -65,8 +65,7 @@ QHash<int, QByteArray> ActivityListModel::roleNames() const
     roles[LinkRole] = "link";
     roles[MessageRole] = "message";
     roles[ActionRole] = "type";
-    roles[DarkIconRole] = "darkIcon";
-    roles[LightIconRole] = "lightIcon";
+    roles[IconRole] = "icon";
     roles[ActionTextRole] = "subject";
     roles[ActionsLinksRole] = "links";
     roles[ActionsLinksContextMenuRole] = "linksContextMenu";
@@ -226,7 +225,7 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
     };
 
     const auto generateIconPath = [&]() {
-        auto colorIconPath = role == DarkIconRole ? QStringLiteral("image://svgimage-custom-color/%1/white") : QStringLiteral("image://svgimage-custom-color/%1/black");
+        auto colorIconPath = QStringLiteral("image://svgimage-custom-color/%1");
         if (a._type == Activity::NotificationType && !a._talkNotificationData.userAvatar.isEmpty()) {
             return QStringLiteral("image://svgimage-custom-color/talk-bordered.svg");
         } else if (a._type == Activity::SyncResultType) {
@@ -250,14 +249,11 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
             } else {
                 // File sync successful
                 if (a._fileAction == "file_created") {
-                    return a._previews.empty() ? QStringLiteral("image://svgimage-custom-color/add.svg/")
-                                               : QStringLiteral("image://svgimage-custom-color/add-bordered.svg/");
+                    return a._previews.empty() ? colorIconPath.arg("add.svg") : colorIconPath.arg("add-bordered.svg");
                 } else if (a._fileAction == "file_deleted") {
-                    return a._previews.empty() ? QStringLiteral("image://svgimage-custom-color/delete.svg/")
-                                               : QStringLiteral("image://svgimage-custom-color/delete-bordered.svg/");
+                    return a._previews.empty() ? colorIconPath.arg("delete.svg") : colorIconPath.arg("delete-bordered.svg");
                 } else {
-                    return a._previews.empty() ? colorIconPath.arg(QStringLiteral("change.svg"))
-                                               : QStringLiteral("image://svgimage-custom-color/change-bordered.svg/");
+                    return a._previews.empty() ? colorIconPath.arg("change.svg") : colorIconPath.arg("change-bordered.svg");
                 }
             }
         } else {
@@ -266,8 +262,8 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
                 return colorIconPath.arg("activity.svg");
             }
 
-            const QString basePath = QStringLiteral("image://tray-image-provider/") % a._icon % QStringLiteral("/");
-            return role == DarkIconRole ? QString(basePath + QStringLiteral("white")) : QString(basePath + QStringLiteral("black"));
+            // using tray-image-provider here as it can read from URLs
+            return QStringLiteral("image://tray-image-provider/%1").arg(a._icon);
         }
     };
 
@@ -275,14 +271,14 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
     case DisplayPathRole:
         return getDisplayPath();
     case PathRole:
-        return QFileInfo(getFilePath()).path();
+        return getFilePath();
     case OpenablePathRole:
-        return a._isMultiObjectActivity ? QFileInfo(getFilePath()).canonicalPath() : QFileInfo(getFilePath()).canonicalFilePath();
+        return getFilePath();
     case DisplayLocationRole:
         return displayLocation();
     case ActionsLinksRole: {
         QList<QVariant> customList;
-        foreach (ActivityLink activityLink, a._links) {
+        for (const auto &activityLink : std::as_const(a._links)) {
             customList << QVariant::fromValue(activityLink);
         }
         return customList;
@@ -296,10 +292,8 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
         return ActivityListModel::convertLinksToActionButtons(a);
     }
 
-    case DarkIconRole:
-    case LightIconRole: {
+    case IconRole:
         return generateIconPath();
-    }
     case ObjectTypeRole:
         return a._objectType;
     case ObjectIdRole:
@@ -313,14 +307,14 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
         case Activity::DummyMoreActivitiesAvailableType:
             return "Activity";
         case Activity::NotificationType:
+        case Activity::OpenSettingsNotificationType:
             return "Notification";
         case Activity::SyncFileItemType:
             return "File";
         case Activity::SyncResultType:
             return "Sync";
-        default:
-            return QVariant();
         }
+        break;
     }
     case ActionTextRole:
         if(a._subjectDisplay.isEmpty()) {
@@ -364,7 +358,8 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
     case IsCurrentUserFileActivityRole:
         return a._isCurrentUserFileActivity;
     case ThumbnailRole: {
-        if (a._type == Activity::NotificationType && !a._talkNotificationData.userAvatar.isEmpty()) {
+        if ((a._type == Activity::NotificationType || a._type == Activity::OpenSettingsNotificationType) &&
+            !a._talkNotificationData.userAvatar.isEmpty()) {
             return generateAvatarThumbnailMap(a._talkNotificationData.userAvatar);
         }
 
@@ -389,7 +384,7 @@ QVariant ActivityListModel::data(const QModelIndex &index, int role) const
         return QVariant::fromValue(a);
     }
 
-    return QVariant();
+    return {};
 }
 
 int ActivityListModel::rowCount(const QModelIndex &parent) const
@@ -602,31 +597,6 @@ void ActivityListModel::addErrorToActivityList(const Activity &activity, const E
     }
 }
 
-void ActivityListModel::addIgnoredFileToList(const Activity &newActivity)
-{
-    qCInfo(lcActivity) << "First checking for duplicates then add file to the notification list of ignored files: " << newActivity._file;
-
-    bool duplicate = false;
-    if (_listOfIgnoredFiles.size() == 0) {
-        _notificationIgnoredFiles = newActivity;
-        _notificationIgnoredFiles._subject = tr("Files from the ignore list as well as symbolic links are not synced.");
-        addEntriesToActivityList({_notificationIgnoredFiles});
-        _listOfIgnoredFiles.append(newActivity);
-        return;
-    }
-
-    foreach (Activity activity, _listOfIgnoredFiles) {
-        if (activity._file == newActivity._file) {
-            duplicate = true;
-            break;
-        }
-    }
-
-    if (!duplicate) {
-        _notificationIgnoredFiles._message.append(", " + newActivity._file);
-    }
-}
-
 void ActivityListModel::addNotificationToActivityList(const Activity &activity)
 {
     qCDebug(lcActivity) << "Notification successfully added to the notification list: " << activity._subject;
@@ -664,9 +634,10 @@ void ActivityListModel::removeActivityFromActivityList(const Activity &activity)
     }
 
     if (activity._type != Activity::ActivityType &&
-            activity._type != Activity::DummyFetchingActivityType &&
-            activity._type != Activity::DummyMoreActivitiesAvailableType &&
-            activity._type != Activity::NotificationType) {
+        activity._type != Activity::DummyFetchingActivityType &&
+        activity._type != Activity::DummyMoreActivitiesAvailableType &&
+        activity._type != Activity::NotificationType &&
+        activity._type != Activity::OpenSettingsNotificationType) {
 
         const auto notificationErrorsListIndex = _notificationErrorsLists.indexOf(activity);
         if (notificationErrorsListIndex != -1)
@@ -715,7 +686,6 @@ void ActivityListModel::slotTriggerDefaultAction(const int activityIndex)
         }
 
         auto folder = FolderMan::instance()->folder(activity._folder);
-        const auto folderDir = QDir(folder->path());
         const auto fileLocation = activity._syncFileItemStatus == SyncFileItem::FileNameInvalidOnServer
             ? InvalidFilenameDialog::FileLocation::NewLocalFile
             : InvalidFilenameDialog::FileLocation::Default;
@@ -724,7 +694,7 @@ void ActivityListModel::slotTriggerDefaultAction(const int activityIndex)
             : InvalidFilenameDialog::InvalidMode::SystemInvalid;
 
         _currentInvalidFilenameDialog = new InvalidFilenameDialog(_accountState->account(), folder,
-            folderDir.filePath(activity._file), fileLocation, invalidMode);
+            folder->filePath(activity._file), fileLocation, invalidMode);
         connect(_currentInvalidFilenameDialog, &InvalidFilenameDialog::accepted, folder, [folder]() {
             folder->scheduleThisFolderSoon();
         });
@@ -735,6 +705,8 @@ void ActivityListModel::slotTriggerDefaultAction(const int activityIndex)
         _currentInvalidFilenameDialog->open();
         ownCloudGui::raiseDialog(_currentInvalidFilenameDialog);
         return;
+    } else if (activity._type == Activity::OpenSettingsNotificationType) {
+        Q_EMIT showSettingsDialog();
     }
 
     if (!path.isEmpty()) {
